library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity pll_det is
	port (
		CLK		: in  std_logic;					-- 73.728MHz/6.144MHz
		RST		: in  std_logic;
		I_IN	: in  std_logic_vector(17 downto 0);
		Q_IN	: in  std_logic_vector(17 downto 0);
		MPX_OUT	: out std_logic_vector(20 downto 0)
		);
end pll_det;

architecture Structural of pll_det is

	component sin_table18
		port (
			CLK		: in  std_logic;
			THETA	: in  std_logic_vector(17 downto 0);
			SIN_OUT	: out std_logic_vector(17 downto 0);
			COS_OUT	: out std_logic_vector(17 downto 0)
			);
	end component;

	component mul_p
		port (
			aclr	: in  std_logic;
			clock	: in  std_logic;
			dataa	: in  std_logic_vector(17 downto 0);
			datab	: in  std_logic_vector(17 downto 0);
			result	: out std_logic_vector(35 downto 0)
			);
	end component;

	signal cnt 			: unsigned(3 downto 0);
	signal theta		: std_logic_vector(17 downto 0);
	signal sin_out		: std_logic_vector(17 downto 0);
	signal cos_out		: std_logic_vector(17 downto 0);
	signal I_INd		: std_logic_vector(17 downto 0);
	signal Q_INd		: std_logic_vector(17 downto 0);
	signal coscos		: std_logic_vector(35 downto 0);
	signal sinsin		: std_logic_vector(35 downto 0);
	signal pd_out0		: std_logic_vector(35 downto 0);
	signal pd_out		: std_logic_vector(35 downto 0);

	signal p_acc		: std_logic_vector(35 downto 0);

begin

	process (CLK, RST)
	begin
		if RST = '1' then
			cnt <= (others => '0');
		elsif CLK'event and CLK = '1' then
			if cnt = 15 then
				cnt <= (others => '0');
			else
				cnt <= cnt + 1;
			end if;
		end if;
	end process;

	process (CLK, RST)
	begin
		if RST = '1' then
			I_INd <= (others => '0');
			Q_INd <= (others => '0');
			p_acc <= (35 => '0', 34 => '0', others => '1');
		elsif CLK'event and CLK = '1' then
			if cnt = 0  then
				I_INd <= I_IN;
				Q_INd <= Q_IN;

			elsif cnt = 6  then
				pd_out0 <= signed(coscos) + signed(sinsin);

			elsif cnt = 7  then
				--A= 1.0
--				if pd_out0(35) /= pd_out0(34) then
--					pd_out <= (35 => pd_out0(35), others => pd_out0(34));
--				else
--					pd_out <= pd_out0(34 downto 0) & '0';
--				end if;
				--A=0.75
				pd_out <= signed(pd_out0(35 downto 0)) + signed(pd_out0(35 downto 1)) + pd_out0(0);

			elsif cnt = 8  then
				p_acc <= signed(p_acc) + signed(pd_out);

			elsif cnt = 9  then
				theta <= signed(p_acc(35 downto 18)) + p_acc(17);
				MPX_OUT <= signed(pd_out(35 downto 15)) + pd_out(14);

			end if;
		end if;
	end process;

	U1 : sin_table18
		port map (
			CLK		=> CLK,
			THETA	=> theta,
			SIN_OUT	=> sin_out,
			COS_OUT	=> cos_out
			);

	U2 : mul_p
		port map (
			aclr	=> RST,
			clock	=> CLK,
			dataa	=> cos_out,
			datab	=> I_INd,
			result	=> coscos
			);

	U3 : mul_p
		port map (
			aclr	=> RST,
			clock	=> CLK,
			dataa	=> sin_out,
			datab	=> Q_INd,
			result	=> sinsin
			);

end Structural;