library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity sin_table18 is
	port (
		CLK		: in  std_logic;
		THETA	: in  std_logic_vector(17 downto 0);
		SIN_OUT	: out std_logic_vector(17 downto 0);
		COS_OUT	: out std_logic_vector(17 downto 0)
		);
end sin_table18;

architecture Behavioral of sin_table18 is

component sin_rom_512_27m
	port (
		address_a	: in  std_logic_vector(8 downto 0);
		address_b	: in  std_logic_vector(8 downto 0);
		clock		: in  std_logic;
		q_a			: out std_logic_vector(26 downto 0);
		q_b			: out std_logic_vector(26 downto 0)
		);
end component;

component sin_rom_512_29s
	port(
		address	: in  std_logic_vector(8 downto 0);
		clock	: in  std_logic;
		q		: out std_logic_vector(28 downto 0)
		);
end component;

component mul2718_p3
	port (
		clock	: in  std_logic;
		dataa	: in  std_logic_vector(26 downto 0);
		datab	: in  std_logic_vector(17 downto 0);
		result	: out std_logic_vector(44 downto 0)
		);
end component;

component mul2711_p3
	port (
		clock	: in  std_logic;
		dataa	: in  std_logic_vector(26 downto 0);
		datab	: in  std_logic_vector(10 downto 0);
		result	: out std_logic_vector(37 downto 0)
		);
end component;

signal addr_ma_a : std_logic_vector(8 downto 0);
signal addr_ma_b : std_logic_vector(8 downto 0);
signal addr_sub : std_logic_vector(8 downto 0);
signal sin_main : std_logic_vector(26 downto 0);
signal cos_main : std_logic_vector(26 downto 0);
signal sin_sub : std_logic_vector(17 downto 0);
signal cos_sub0 : std_logic_vector(10 downto 0);
signal sincos0 : std_logic_vector(37 downto 0);
signal cossin0 : std_logic_vector(44 downto 0);
signal coscos0 : std_logic_vector(37 downto 0);
signal sinsin0 : std_logic_vector(44 downto 0);
signal cossin1 : std_logic_vector(26 downto 0);
signal sinsin1 : std_logic_vector(26 downto 0);
signal sin_main1   : std_logic_vector(26 downto 0);
signal sin_main1d1 : std_logic_vector(26 downto 0);
signal sin_main1d2 : std_logic_vector(26 downto 0);
signal cos_main1   : std_logic_vector(26 downto 0);
signal cos_main1d1 : std_logic_vector(26 downto 0);
signal cos_main1d2 : std_logic_vector(26 downto 0);
signal srcos : std_logic_vector(26 downto 0);
signal crcos : std_logic_vector(26 downto 0);
signal sin0 : std_logic_vector(26 downto 0);
signal sin1 : std_logic_vector(26 downto 0);
signal cos0 : std_logic_vector(26 downto 0);
signal cos1 : std_logic_vector(26 downto 0);

begin

	addr_ma_a <= THETA(17 downto 9);
	addr_ma_b <= unsigned(THETA(17 downto 9)) + 128;
	addr_sub <= THETA(8 downto 0);

	U1: sin_rom_512_27m
		port map(
			address_a	=> addr_ma_a,
			address_b	=> addr_ma_b,
			clock		=> CLK,
			q_a			=> sin_main,
			q_b			=> cos_main
			);

	U2: sin_rom_512_29s
		port map(
			address		=> addr_sub,
			clock		=> CLK,
			q(28 downto 11)	=> sin_sub,
			q(10 downto  0)	=> cos_sub0
			);

	U3: mul2711_p3
		port map(
			clock	=> CLK,
			dataa	=> sin_main,
			datab	=> cos_sub0,
			result	=> sincos0
			);

	U4: mul2718_p3
		port map(
			clock	=> CLK,
			dataa	=> cos_main,
			datab	=> sin_sub,
			result	=> cossin0
			);

	U5: mul2711_p3
		port map(
			clock	=> CLK,
			dataa	=> cos_main,
			datab	=> cos_sub0,
			result	=> coscos0
			);

	U6: mul2718_p3
		port map(
			clock	=> CLK,
			dataa	=> sin_main,
			datab	=> sin_sub,
			result	=> sinsin0
			);

	srcos(26 downto 15) <= (others => sincos0(37)); srcos(14 downto 0) <= sincos0(37 downto 23);
	crcos(26 downto 15) <= (others => coscos0(37));	crcos(14 downto 0) <= coscos0(37 downto 23);
	cossin1(26 downto 22) <= (others => cossin0(44)); cossin1(21 downto 0) <= cossin0(44 downto 23);
	sinsin1(26 downto 22) <= (others => sinsin0(44)); sinsin1(21 downto 0) <= sinsin0(44 downto 23);

	process(CLK)
	begin
		if CLK'event and CLK = '1' then
			cos_main1 <= signed(cos_main) - SHR(signed(cos_main), "110");
			sin_main1 <= signed(sin_main) - SHR(signed(sin_main), "110");
			cos_main1d1 <= cos_main1;
			sin_main1d1 <= sin_main1;
			cos_main1d2 <= cos_main1d1;
			sin_main1d2 <= sin_main1d1;
			--------------------------------------
			COS_OUT <= signed(cos1(26 downto 9)) + cos1(8);
			SIN_OUT <= signed(sin1(26 downto 9)) + sin1(8);
		end if;
	end process;

	sin0 <= signed(sin_main1d2) - signed(srcos);
	cos0 <= signed(cos_main1d2) - signed(crcos);
	sin1 <= signed(sin0) + signed(cossin1);
	cos1 <= signed(cos0) - signed(sinsin1);

end Behavioral;