library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.math_real.all;
use std.textio.all;
use ieee.std_logic_textio.all;

entity tb1 is
end tb1;
 
architecture behavior of tb1 is 

	component pll_det port(
		CLK		: in  std_logic;					-- 73.728MHz/ 6.144MHz
		RST		: in  std_logic;
		I_IN	: in  std_logic_vector(17 downto 0);
		Q_IN	: in  std_logic_vector(17 downto 0);
		MPX_OUT	: out std_logic_vector(20 downto 0)
		);
	end component;

	--Inputs
	signal CLK : std_logic := '0';
	signal RST : std_logic := '0';
	signal i_sig : std_logic_vector(17 downto 0);
	signal q_sig : std_logic_vector(17 downto 0);

	--Outputs
	signal MPX_OUT : std_logic_vector(20 downto 0);

	-- Clock period definitions
	constant CLK_period : time := 162.76 ns;
--	constant CLK_period : time := 13.5634 ns;
	constant pai : real := 3.1415926536;
	signal cfreq : real := 384000.0;

	signal cnt : unsigned(3 downto 0);
--	signal cnt : unsigned(7 downto 0);
	signal en : std_logic;
	signal aacc : real := 0.0;
	signal audio : real := 0.0;
	signal pacc : real := 0.0;
	signal freq : real := 1000.0;
	signal fdev : real := 70000.0;

begin
 
	-- Instantiate the Unit Under Test (UUT)
	uut: pll_det port map(
		CLK => CLK,
		RST => RST,
		I_IN => i_sig,
		Q_IN => q_sig,
		MPX_OUT => MPX_OUT
		);

	-- CLK: clock generation
	process
	begin
		CLOCK_LOOP : loop
			CLK <= '1';
			wait for CLK_period/2;
			CLK <= '0';
			wait for CLK_period/2;
		end loop CLOCK_LOOP;
	end process;

 	-- RST: asserting reset
	process
	begin
		RST <= '0';
		wait for CLK_period*0.2;
		RST <= '1';
		wait for CLK_period;
		RST <= '0';
		wait;
	end process;

	-- Divide by 16 counter to generate 384k
	process(CLK, RST)
	begin
		if RST = '1' then
			cnt <= (others => '0');
		elsif CLK'event and CLK = '1' then
			en <= '0';
			if cnt = 15 then
--			if cnt = 191 then
				en <= '1';
				cnt <= (others => '0');
			else
				cnt <= cnt + 1;
			end if;
		end if;
	end process;

	-- Audio NCO
	process(CLK, RST)
	variable tmp : real := 0.0;
	begin
		if RST = '1' then
			aacc <= 0.0;
		elsif CLK'event and CLK = '1' then
			if en = '1' then
				tmp := aacc + freq/cfreq*2.0;
				if tmp > 1.0 then
					tmp := tmp - 2.0;
				elsif tmp < -1.0 then
					tmp := tmp + 2.0;
				end if;
				aacc <= tmp;
			end if;
		end if;
	end process;

	audio <= sin(pai*aacc);

	-- Generator NCO
	process(CLK, RST)
	variable tmp : real := 0.0;
	begin
		if RST = '1' then
			pacc <= 0.0;
		elsif CLK'event and CLK = '1' then
			if en = '1' then
				tmp := pacc + audio*fdev/cfreq*2.0;
				if tmp > 1.0 then
					tmp := tmp - 2.0;
				elsif tmp < -1.0 then
					tmp := tmp + 2.0;
				end if;
				pacc <= tmp;
			end if;
		end if;
	end process;

	i_sig <= std_logic_vector(conv_signed(integer(cos(pai*pacc)*65536.0 + 0.5), 18));
	q_sig <= std_logic_vector(conv_signed(integer(sin(pai*pacc)*65536.0 + 0.5), 18));

end;